C
Code by Michael Kupferschmid
C
C     This program reads numbers from standard-in, adds them,
C     and writes their sum to standard-out.  It is intended for
C     use in a pipeline.
C
C     variable  meaning
C     --------  -------
C     BUFFER    to hold output for blank removal
C     DFLOAT    Fortran function returns REAL*8 for INTEGER*4
C     EXIT      system routine stops program with a return code
C     FRDR8V    routine reads unknown-length line of REAL*8 values
C     I         index on numbers read from this line
C     IFIX      Fortran function returns INTEGER*4 for REAL*4
C     K         index on the characters of BUFFER
C     L         length of BUFFER
C     LDR       dimensioned size of R
C     LENGTH    function returns index of last nonblank in a string
C     N         number of numbers found in input line
C     R         a vector of numbers read in
C     RC        return code from FRDR8V; 0 => all went well
C     SHIFTL    routine removes leading blanks from a string
C     INT8      Fortran function returns INTEGER*8 for REAL*8
C     SUM       accumulated amount
C
      PARAMETER(LDR=256)
      REAL*8 R(LDR),SUM
      INTEGER*4 RC
      CHARACTER*32 BUFFER/'                                '/
C
C ------------------------------------------------------------------
C
C     read and add numbers from standard-in until EOF
      SUM=0.D0
    4 CALL FRDR8V(5,R,LDR, N,RC)
      IF(RC.EQ.1) GO TO 1
      IF(RC.NE.0) GO TO 2
      IF(N.EQ.0) GO TO 4
      DO 3 I=1,N
           SUM=SUM+R(I)
    3 CONTINUE
      GO TO 4
C
C     on EOF write the sum
    1 IF(DFLOAT(INT8(SUM)).EQ.SUM) THEN
C       there is no fractional part
        WRITE(BUFFER,*) INT8(SUM)
      ELSE
C       there is a fractional part
        WRITE(BUFFER,*) SUM
      ENDIF
C
C     remove leading blanks
      CALL SHIFTL(BUFFER,24)
      L=LENGTH(BUFFER,24)
      WRITE(6,900) (BUFFER(K:K),K=1,L)
  900 FORMAT(24A1)
      CALL EXIT(0)
C
C     handle input errors
    2 WRITE(0,901)
  901 FORMAT('add: input error')
      CALL EXIT(1)
      END
