C
Code by Michael Kupferschmid
C
C     This program compares two files of real numbers that are
C     thought to be approximately the same.
C
C     variable  meaning
C     --------  -------
C     A         a vector of numbers read from file A
C     ARG       a command-line argument
C     B         a vector of numbers read from file B
C     DABS      Fortran function returns |REAL*8|
C     DIFF      absolute difference between A and B
C     DIFFMX    maximum absolute difference found
C     DMAX1     Fortran function returns larger of REAL*8s
C     FINDST    routine finds starting location of a substring
C     FRDR8V    routine reads unknown-length line of REAL*8 values
C     GETARG    system routine gets one command-line argument 
C     GETFIL    routine attaches a file for a given purpose
C     I         index on numbers in a line
C     IARGC     system function for number of command-line arguments
C     IMAX      maximum number of numbers per line
C     J         index on command-line arguments
C     KSTART    starting index of "=" in ARG, or zero
C     L         length of a command-line argument
C     LENGTH    function returns index of last nonblank in a string
C     LNUM      number of the line(s) just read in
C     NA        number of numbers found on file A line
C     NARG      number of command-line arguments found
C     NB        number of numbers found on file B line
C     NERR      number of differences out of tolerance
C     RC        return code; 0 => all went well
C     TOL       maximum allowed difference between the numbers
C
      PARAMETER(IMAX=10)
      REAL*8 A(IMAX),B(IMAX),DIFF,DIFFMX,TOL
      INTEGER*4 RC
      CHARACTER*24 ARG
C
C ------------------------------------------------------------------
C
C     attach the files
      CALL GETFIL('file A',6,' ',0,1,1)
      CALL GETFIL('file B',6,' ',0,2,1)
      CALL GETFIL('differences',11,' ',0,3,2)
      LNUM=1
      DIFFMX=0.D0
      NERR=0
C
C     look for a parameter specifying a difference tolerance
      NARG=IARGC()
      IF(NARG.GT.0) THEN
         DO 1 J=1,NARG
              CALL GETARG(J,ARG)
              L=LENGTH(ARG,24)
C             skip if it is a unit assignment
              CALL FINDST(ARG,L,'=',1, KSTART)
              IF(KSTART.EQ.0) THEN
                 READ(ARG,*,ERR=1) TOL
                 GO TO 2
              ENDIF
    1    CONTINUE
      ENDIF
      TOL=1.D-09
    2 WRITE(6,901) TOL
  901 FORMAT('using difference tolerance ',1PD13.6)
C
C     get the next set of numbers to compare
    6 CALL FRDR8V(1,A,IMAX, NA,RC)
      IF(RC.EQ.1) GO TO 3
      IF(RC.NE.0) THEN
         WRITE(0,902) RC,LNUM
  902    FORMAT('RC=',I1,' reading file A line',I10)
         STOP
      ENDIF
      CALL FRDR8V(2,B,IMAX, NB,RC)
      IF(RC.EQ.1) GO TO 4
      IF(RC.NE.0) THEN
         WRITE(0,903) RC,LNUM
  903    FORMAT('RC=',I1,' reading file B line',I10)
         STOP
      ENDIF
      IF(NA.NE.NB) THEN
         WRITE(0,904) NA,NB,LNUM
  904    FORMAT('NA=',I2,' <> ',I2,'=NB at line',I10)
         STOP
      ENDIF
C
C     report differences greater than the tolerance
      DO 5 I=1,NA
           DIFF=DABS(A(I)-B(I))
           DIFFMX=DMAX1(DIFFMX,DIFF)
           IF(DIFF.GT.TOL) THEN
              WRITE(3,905) LNUM,I,(A(I)-B(I))
  905         FORMAT('line',I10,' tokens ',I2,' differ by ',1PE13.6)
              NERR=NERR+1
           ENDIF
    5 CONTINUE
      LNUM=LNUM+1
      GO TO 6
C
C     end of A first; also at end of B?
    3 READ(2,*,END=7)
C     end of B first (=>A longer) or end of A but not B (=>B longer)
    4 WRITE(0,906)
  906 FORMAT('the files are of unequal lengths')
C
C     report whether differences out of tolerance were found
    7 IF(NERR.GT.0) THEN
         WRITE(6,907) NERR,DIFFMX
  907    FORMAT('differences file lists ',I10,
     ;          ' differences, largest difference ',1PE13.6)
      ELSE
         WRITE(6,908) LNUM-1,DIFFMX
  908    FORMAT('files compared equal within tolerance,',I10,
     ;          ' lines, largest difference ',1PE13.6)
      ENDIF
      STOP
      END
