C
Code by Michael Kupferschmid
C
C     This routine reads x-y data from standard-in and writes
C     the boxcar average on standard-out.  The width of the
C     boxcar is given as a command-line parameter.
C
C     variable  quantity
C     --------  --------
C     ARG       run-line argument as one long word
C     DABS      Fortran function returns |REAL*8|
C     DFLOAT    Fortran function returns REAL*8 for INTEGER*4
C     EOF       T => the end of the data has been reached
C     EXIT      system routine stops program with a return code
C     GETARG    system routine gets one command-line argument
C     I         index in X and Y of most recent data point
C     IARGC     system function for number of command-line arguments
C     IMAX      largest value of I to use in boxcar
C     J         index in X and Y of a data point in the boxcar
C     LNUM      number of line just read from unit 5
C     N         number of values in the boxcar
C     NARGS     number of command-line arguments supplied
C     W         width of the boxcar in X
C     X         recent X values of input data
C     XAVG      average of X values in the boxcar
C     XDIFF     difference between X values now in boxcar
C     XSUM      sum of X values in the boxcar
C     Y         recent Y values of input data
C     YAVG      average of Y values in the boxcar
C     YSUM      sum of Y values in the boxcar
C
C     prepare to get and process the boxcar width
      CHARACTER*23 ARG
      REAL*8 W
C
C     prepare to read and process the data
      LOGICAL*4 EOF
      INTEGER*4 IMAX
      PARAMETER(IMAX=100)
      REAL*8 X(IMAX),Y(IMAX),XDIFF,XSUM,XAVG,YSUM,YAVG
C
C ------------------------------------------------------------------
C
C     find out how wide the boxcar should be
      NARGS=IARGC()
      IF(NARGS.EQ.0) THEN
         WRITE(0,901)
  901    FORMAT('usage: cat infile | boxcar width > outfile')
         CALL EXIT(1)
      ENDIF
      CALL GETARG(1,ARG)
      READ(ARG,*,ERR=1) W
      IF(W.LE.0.D0) GO TO 1
      GO TO 2
    1 WRITE(0,902) ARG
  902 FORMAT('bogus boxcar width: ',A23)
      CALL EXIT(2)
C
C     initialize
    2 EOF=.FALSE.
      LNUM=1
      I=0
C
C     read the next data point
    8 I=I+1
      IF(I.GT.IMAX) THEN
         WRITE(0,903) IMAX
  903    FORMAT('working vectors too short at length ',I4)
         CALL EXIT(3)
      ENDIF
      READ(5,*,END=3,ERR=4) X(I),Y(I)
      LNUM=LNUM+1
      IF(I.GT.1) THEN
         IF(X(I).LT.X(I-1)) THEN
            WRITE(0,904)
  904       FORMAT('input X values are not non-decreasing')
            CALL EXIT(4)
         ENDIF
      ENDIF
      GO TO 5
    3 EOF=.TRUE.
      WRITE(0,905) XDIFF
  905 FORMAT('final boxcar has width ',1PD23.16)
      GO TO 6
C
C     accumulate sums until the end of the boxcar is reached
    5 N=0
      XSUM=0.D0
      YSUM=0.D0
      DO 7 J=1,I
           XDIFF=DABS(X(J)-X(1))
           IF(XDIFF .GT. W) GO TO 6
           N=N+1
           XSUM=XSUM+X(J)
           YSUM=YSUM+Y(J)
    7 CONTINUE
C     ready for more data
      GO TO 8
C
C     average over the boxcar and report the result
    6 XAVG=XSUM/DFLOAT(N)
      YAVG=YSUM/DFLOAT(N)
      WRITE(6,906) XAVG,YAVG
  906 FORMAT(1PD23.16,1X,1PD23.16)
      IF(EOF) CALL EXIT(0)
C
C     move the boxcar forward by 1 data point
      DO 9 J=1,I-1
           X(J)=X(J+1)
           Y(J)=Y(J+1)
    9 CONTINUE
      I=I-1
      GO TO 5
C
C     report bad input data
    4 WRITE(0,907) LNUM
  907 FORMAT('bad input data at line ',I4)
      CALL EXIT(5)
      END
