C
Code by Michael Kupferschmid
C
C     This program reads a list of observations from standard-in
C     and writes coordinates (TAU,CDF) on standard-out for plotting
C     the cumulative distribution function of the data,
C
C          CDF(TAU) = frequency{X <= TAU}
C
C     variable  meaning
C     --------  -------
C     CDF       a cumulative distribution function value
C     DATA      an input data value
C     DFLOAT    Fortran function returns REAL*8 for INTEGER*4
C     EXIT      system routine stops program with a return code
C     I         index on the data values
C     K         cumulative number of observations
C     N         number of observations
C     NMAX      maximum allowed number of observations
C     SORTR8    routine sorts a REAL*8 vector in ascending order
C     TAU       cutoff value of random variable
C     X         the vector of observations
C
      PARAMETER(NMAX=1000)
      REAL*8 CDF,DATA,TAU,X(NMAX)
C
C ------------------------------------------------------------------
C
C     get the observations
      N=0
    2 READ(5,*,END=1) DATA
      N=N+1
      IF(N.LE.NMAX) THEN
         X(N)=DATA
         GO TO 2
      ELSE
         WRITE(0,901) NMAX
  901    FORMAT('error: more than ',I4,' observations')
         CALL EXIT(1)
      ENDIF
C
C     sort the data in increasing order
    1 CALL SORTR8(N,X)
C
C     output coordinates for plotting the square wave of the CDF
      K=0
      CDF=0.D0
      DO 3 I=1,N
           K=K+1
           TAU=X(I)
           WRITE(6,902) TAU,CDF
           CDF=DFLOAT(K)/DFLOAT(N)
           WRITE(6,902) TAU,CDF
  902      FORMAT(2(1X,1PE13.6))
    3 CONTINUE
      CALL EXIT(0)
      END
