C
Code by Michael Kupferschmid
C
C     This program reports the fraction closest to a given decimal.
C
C     variable  meaning
C     --------  -------
C     ARG       a command-line argument as characters
C     DABS      Fortran function returns |REAL*8|
C     DBEST     best denominator found so far
C     DEN       trial denominator
C     DENMAX    largest denominator to consider
C     DFLOAT    Fortran function returns REAL*8 for INTEGER*4
C     ERR       current error
C     ERRMIN     best error found so far
C     EXIT      system routine stops program with a return code
C     GETARG    system routine gets one command-line argument
C     I         index on trial DEN values
C     IARGC     system function for number of command-line arguments
C     IFIX      Fortran function returns INTEGER*4 for REAL*4
C     NARG      number of command-line arguments found
C     NBEST     best numerator found so far
C     NUM       a numerator
C     SNGL      Fortran function returns REAL*4 for REAL*8
C     T         whole-number part of X
C     X         decimal number to find closest fraction to
C     Y         decimal value of trial fraction
C
      CHARACTER*19 ARG
      INTEGER*4 DBEST,DEN,DENMAX
      REAL*8 ERR,ERRMIN/1.D+10/,T,X,Y
C
C ------------------------------------------------------------------
C
C     tell how to use the program
      NARG=IARGC()
      IF(NARG.LT.1 .OR. NARG.GT.2) THEN
         WRITE(0,901)
  901    FORMAT('usage: closest .1234 [maxden]'/
     ;          'prints rational fraction closest to given decimal')
         CALL EXIT(1)
      ENDIF
C
C     get a decimal fraction from the command line
      CALL GETARG(1,ARG)
      READ(ARG,*,ERR=1) X
      IF(X.LE.0.D0 .OR. X.GE.1.D0) THEN
         WRITE(0,902)
  902    FORMAT('decimal fraction must be in (0,1)')
         CALL EXIT(2)
      ENDIF
C
C     get the biggest denominator to be considered
      IF(NARG.EQ.2) THEN
         CALL GETARG(2,ARG)
         READ(ARG,*,ERR=1) DENMAX
         IF(DENMAX.LE.0) THEN
            WRITE(0,903)
  903       FORMAT('maximum denominator must be positive')
            CALL EXIT(3)
         ENDIF
      ELSE
C        otherwise give it this default value
         DENMAX=999
      ENDIF
C
C     consider progressively bigger denominators
      DO 2 I=1,DENMAX
           DEN=I
C
C          find NUM to make NUM/DEN = X
           T=X*DFLOAT(DEN)
           NUM=IFIX(SNGL(T)+0.5)
C
C          find the error in that approximation to X
           Y=DFLOAT(NUM)/DFLOAT(DEN)
           ERR=DABS(Y-X)
C
C          remember the closest fraction found so far
           IF(ERR.LT.ERRMIN) THEN
              ERRMIN=ERR
              NBEST=NUM
              DBEST=DEN
           ENDIF
    2 CONTINUE
      Y=DFLOAT(NBEST)/DFLOAT(DBEST)
      ERR=Y-X
      WRITE(6,904) NBEST,DBEST,Y,X,ERR
  904 FORMAT(I3,'/',I3,' = ',F7.5,' - ',F7.5,' = ',1PE13.6)
      CALL EXIT(0)
C
C     report parameter errors
    1 WRITE(0,905) ARG
  905 FORMAT('parameter ',A19,' is in error')
      CALL EXIT(4)
      END
