C
Code by Michael Kupferschmid
C
C     This program plots a histogram from data.
C
C     unit  use
C     ----  -----------
C      0    error messages
C      5    input data
C      6    output screen display of histogram
C      9    plotting data to /tmp/pdf and /tmp/cdfi
C
C     variable  meaning
C     --------  -------
C     BINS      contents of command-line parameter if any
C     CDFI      cumulative distribution function
C     COLS      contents of $COLUMNS shell variable
C     DFLOAT    Fortran function gives REAL*8 for INTEGER*4
C     DTB       routine converts numerals to an INTEGER*4
C     EXIT      system routine stops program with a return code
C     GETARG    system routine gets one command-line argument
C     GETENV    system routine gets a shell variable if present
C     HIST      routine computes histogram
C     I         index on the observations
C     IARGC     system function for number of command-line arguments
C     INTERP    function interpolates in a table
C     J         index on the cells of the histogram or CDF
C     L         index on data values for plotting CDFI
C     LAST      previous cell used by INTERP
C     MARGIN    width of display screen in characters
C     MXOBS     maximum number of observations allowed
C     MXPDF     maximum number of cells allowed in histogram
C     NARG      number of command-line arguments found
C     NC        cells; 0 => automatic; <0 => allow empty cells
C     NOBS      number of observations found
C     OBS       an observation
C     PDF       probability density function
C     PRNHST    routine prints histogram
C     RC        return code; 0 => all went well
C     RCD       return code from FRDR8V; 0 => not done yet
C     SHIFTR    routine makes string flush right in its length
C     U         a CDF value between 0 and 1
C     X         the vector of observations
C     Y         a function value found by interpolation
C
      PARAMETER(MXOBS=30000,MXPDF=17)
C     MXPDF=NC+1 where NC=1+ceil(lg(MXOBS))
C
      REAL*8 X(MXOBS),PDF(MXPDF,2),CDFI(MXPDF,2)
      REAL*8 OBS,U,Y,INTERP
      INTEGER*4 RC,RCD
      CHARACTER*3 BINS,COLS
C
C ------------------------------------------------------------------
C
C     input the data set, one number per line
      I=0
    3 READ(5,*,END=1,ERR=2) OBS
      IF(I.LT.MXOBS) THEN
         I=I+1
         X(I)=OBS
         NOBS=I
         GO TO 3
      ELSE
         WRITE(0,991) MXOBS
  991    FORMAT('more than ',I5,' observations')
         CALL EXIT(1)
      ENDIF
    2 WRITE(0,992) I+1
  992 FORMAT('bad data at line ',I5)
      CALL EXIT(2)
C
C     allow the user to override automatic bin determination
    1 NC=0 
      NARG=IARGC()
      IF(NARG.EQ.1) THEN
         CALL GETARG(1,BINS)
         CALL SHIFTR(BINS,3)
         CALL DTB(BINS,3, NC,RC)
         IF(RC.NE.0 .OR. NC.LT.0) NC=0
      ENDIF
C
C     compute the histogram
      CALL HIST(X,NOBS,MXPDF, NC, PDF,CDFI,RC)
      IF(RC.NE.0) THEN
         WRITE(0,993) RC
  993    FORMAT('RC=',I1,' from HIST')
         CALL EXIT(3)
      ENDIF
C
C     find out how wide the display is
      CALL GETENV('COLUMNS',COLS)
      CALL SHIFTR(COLS,3)
      CALL DTB(COLS,3,MARGIN,RC)
      IF(RC.NE.0 .OR. MARGIN.LE.0) MARGIN=80
C
C     print the histogram on the screen
      CALL PRNHST(6,' ',0,NOBS,PDF,MXPDF,NC,MARGIN,RC)
      IF(RC.NE.0) THEN
         WRITE(0,994) RC
  994    FORMAT('RC=',I1,' from PRNHST')
         CALL EXIT(4)
      ENDIF
C
C     write coordinates for the PDF
      OPEN(UNIT=9,FILE='/tmp/pdf')
      WRITE(9,901) PDF(1,1),0.D0
  901 FORMAT(2(1X,1PE13.6))
      DO 4 J=2,NC+1
           WRITE(9,901) PDF(J-1,1),PDF(J-1,2)
           WRITE(9,901) PDF(J,1),PDF(J-1,2)
    4 CONTINUE
      WRITE(9,901) PDF(NC+1,1),0.D0
      CLOSE(UNIT=9)
C
C     write coordinates for the inverse CDF
      OPEN(UNIT=9,FILE='/tmp/cdfi')
      LAST=0
      DO 5 L=1,101
           U=0.01D0*DFLOAT(L-1)
           Y=INTERP(U,CDFI,MXPDF,NC+1,LAST)
           WRITE(9,901) U,Y
    5 CONTINUE
      CLOSE(UNIT=9)
      CALL EXIT(0)
      END
