C
Code by Michael Kupferschmid
C
C     This program generates LaTeX source for typesetting
C     a pie chart.
C
C     unit  purpose
C     ----  -------
C       0   prompts and error messages
C       1   input data
C       9   output LaTeX source
C
C     variable  meaning
C     --------  -------
C     ANGLE     angle of a line in radians
C     DCOS      Fortran function returns cosine of a REAL*8
C     DESCS     descriptions of all items
C     DSIN      Fortran function returns sine of a REAL*8
C     DX        horizontal projection of a line
C     DY        vertical projection of a line
C     EXIT      system routine stops program with a return code
C     FN        name of input data file, then output data file
C     GETFIL    routine attaches a file for a given purpose
C     I         index on items
C     IABS      Fortran function returns |INTEGER*4|
C     IROUND    function returns integer nearest a REAL*8
C     L         length of a string
C     LENGTH    function returns index of last nonblank in a string
C     LINE      a line of data read from the input file
C     LNUM      number of data line just read
C     MXITM     maximum number of items allowed
C     N         number of items
C     QTY       quantity of an item
C     QTYS      quantities of all item
C     R         length of a line
C     RC        return code from SHFTKN
C     SHFTKN    routine shifts contents of a string left by tokens
C     SUM       sum of quantities
C     TWOPI     twice the circle constant
C     XS        x-coordinates of the labels [points]
C     YS        y-coordinates of the labels [points]
C
C     prepare to read and parse input lines
      CHARACTER*80 LINE
      PARAMETER(MXITM=30)
      REAL*8 QTY,QTYS(MXITM)
      INTEGER*4 XS(MXITM),YS(MXITM),RC
      CHARACTER*30 DESCS(MXITM)
C
C     prepare to generate output lines
      CHARACTER*24 FN
      REAL*8 SUM,ANGLE,TWOPI/6.2831853071795862D0/
      INTEGER*4 DX,DY,R
C
C ------------------------------------------------------------------
C
C     prepare to read the data describing the pie chart
      CALL GETFIL('data',4,' ',0,1,1)
      SUM=0.D0
      LNUM=0
      N=0
C
C     read and parse the lines of data
    2 READ(1,901,END=1) LINE
  901 FORMAT(A80)
      LNUM=LNUM+1
      L=LENGTH(LINE,80) 
      IF(L.EQ.0) GO TO 2
      N=N+1
      IF(N.GT.MXITM) THEN
         WRITE(0,991) MXITM
  991    FORMAT('more than',I3,' data items found')
         CALL EXIT(1)
      ENDIF
      READ(LINE,*,ERR=3) QTY
      QTYS(N)=QTY
      SUM=SUM+QTY
      CALL SHFTKN(LINE,L,1,RC)
      IF(RC.NE.0) GO TO 3
      L=LENGTH(LINE,L)
      READ(LINE,*,ERR=3) XS(N)
      CALL SHFTKN(LINE,L,1,RC)
      IF(RC.NE.0) GO TO 3
      L=LENGTH(LINE,L)
      READ(LINE,*,ERR=3) YS(N)
      CALL SHFTKN(LINE,L,1,RC)
      IF(RC.NE.0) GO TO 3
      L=LENGTH(LINE,L)
      IF(L.GT.30) GO TO 3
      DESCS(N)=LINE
      GO TO 2
C
C     construct a suggested output file name
    1 INQUIRE(UNIT=1,NAME=FN)
      L=LENGTH(FN,24)
      IF(L.LE.20) THEN
         FN(L+1:L+1)='.'
         FN(L+2:L+2)='t'
         FN(L+3:L+3)='e'
         FN(L+4:L+4)='x'
         L=L+4
      ELSE
         FN='                        '
         L=0
      ENDIF
      
C     generate LaTeX commands on unit 9
      CALL GETFIL('LaTeX',5,FN,L,9,2)
      WRITE(9,902)
  902 FORMAT('\documentclass{article}',
     ;       '  % remember to delete %%EOF'/
     ;       '\pagestyle{empty}',
     ;       '        % from the last line of the .ps file'/
     ;       '\usepackage{fullpage}',
     ;       '    % if you will \includegraphics it'/
     ;       '\usepackage{pict2e}'/
     ;       '\usepackage{graphicx}'/
     ;       '\begin{document}'/
     ;       '\Large'/
     ;       '\noindent\begin{picture}(470,470)'/
     ;       '\put(235,235){\circle{470}}')
      ANGLE=0.D0
      DO 5 I=1,N
           ANGLE=ANGLE+TWOPI*(QTYS(I)/SUM)
           DX=IROUND(235.D0*DCOS(ANGLE))
           DY=IROUND(235.D0*DSIN(ANGLE))
           IF(DX.NE.0) THEN
              R=IABS(DX)
           ELSE
              R=IABS(DY)
           ENDIF
           WRITE(9,903) DX,DY,R
  903      FORMAT('\put(235,235){\line(',I4,',',I4,'){',I3,'}}')
           WRITE(9,904) XS(I),YS(I),DESCS(I)
  904      FORMAT('\put(',I4,',',I4,'){',A30,'}')
    5 CONTINUE
      WRITE(9,905)
  905 FORMAT('\end{picture}'/
     ;       '\end{document}')
      CALL EXIT(0)
C
C     report input errors
    3 WRITE(0,992) LNUM
  992 FORMAT('bogus data on line',I3)
      CALL EXIT(2)
      END
