C
Code by Michael Kupferschmid
C
C     This program fits a straight line to (x,y) data from
C     standard in, and reports the fit.
C
C     unit  purpose
C     ----  -------
C       0   error and informational messages
C       5   input coordinate pairs
C       6   output equations of best-fit lines
C       9   plotting coordinates
C
C     variable  meaning
C     --------  -------
C     ARG       first command-line argument if any
C     BETA      coefficients in linear fit y=BETA(1)+BETA(2)*x
C     EXIT      system routine stops program with a return code
C     GETARG    system routine gets one command-line argument
C     I         index on data pairs read in
C     IARGC     sustem function for number of command-line arguments
C     L1REGW    routine does weighted least-abs-values regression
C     L2REGW    routine does weighted linear regression
C     N         number of data pairs read in
C     NARG      number of command-line arguments found
C     NMAX      maximum number of data points allowed
C     PLOT      T => write plotting coordinates
C     RC1       return code from L1REGW
C     RC2       return code from L2REGW
C     W         weights (all 1 in this program) 
C     X         the x values
C     XIN       an x value
C     Y         the y values
C     YIN       a y value
C
      PARAMETER(NMAX=1000)
      REAL*8 X(NMAX),Y(NMAX),W(NMAX),BETA(2),XIN,YIN
      INTEGER*4 RC1,RC2
      CHARACTER*4 ARG
      LOGICAL*4 PLOT
C
C ------------------------------------------------------------------
C
C     read and count (x,y) pairs from standard-in
      DO 1 I=1,NMAX
           READ(5,*,END=2,ERR=3) XIN,YIN
           N=I
           X(N)=XIN
           Y(N)=YIN
           W(N)=1.D0
    1 CONTINUE
      WRITE(0,991) NMAX
  991 FORMAT('more than',I5,' data pairs')
      CALL EXIT(1)
    3 WRITE(0,992) N+1
  992 FORMAT('read error at line',I5)
      CALL EXIT(2)
    2 IF(N.LT.2) THEN
         WRITE(0,993) N
  993    FORMAT('N=',I1,' does not permit a regression')
         CALL EXIT(3)
      ENDIF
C
C     find out if plotting coordinates are to be written
      NARG=IARGC()
      IF(NARG.EQ.1) THEN
         CALL GETARG(1,ARG)
         PLOT=(ARG.EQ.'plot')
      ENDIF
C
C     compute the least-squares regression line
      CALL L2REGW(X,Y,W,N, BETA,RC2)
      IF(RC2.EQ.0) THEN
         WRITE(6,901) BETA(1),BETA(2)
  901    FORMAT('L2: y=[',1PE13.6,'] + [',1PE13.6,']x')
         IF(PLOT) THEN
            OPEN(UNIT=9,FILE='/tmp/l2data')
            DO 4 I=1,N
                 WRITE(9,902) X(I),BETA(1)+X(I)*BETA(2)
  902            FORMAT(2(1X,1PE13.6))
    4       CONTINUE
         ENDIF
      ELSE
         WRITE(6,994) RC2
  994    FORMAT('least squares regression failed, RC=',I2)
      ENDIF
C
C     compute the least-absolute-value regression line
      CALL L1REGW(X,Y,W,N, BETA,RC1)
      IF(RC1.EQ.0) THEN
         WRITE(6,903) BETA(1),BETA(2)
  903    FORMAT('L1: y=[',1PE13.6,'] + [',1PE13.6,']x')
         IF(PLOT) THEN
            OPEN(UNIT=9,FILE='/tmp/l1data')
            DO 5 I=1,N
                 WRITE(9,902) X(I),BETA(1)+X(I)*BETA(2)
    5       CONTINUE
         ENDIF
      ELSE
         WRITE(6,995) RC1
  995    FORMAT('least absolute values regression failed, RC=',I2)
      ENDIF
C
C     report if plotting data was written
      IF(PLOT .AND. (RC2.EQ.0 .OR. RC1.EQ.0)) WRITE(0,904)
  904 FORMAT('plotting data was left in /tmp')
      IF(RC2.NE.0) CALL EXIT(4)
      IF(RC1.NE.0) CALL EXIT(5)
      CALL EXIT(0)
      END
