C
Code by Michael Kupferschmid
C
C     This routine does the one-shot time calculation
C     specified in the command-line parameter.
C
C     Times are specified as hh:mm or as minutes
C     tcalc 2:37 prints 0:157
C     tcalc 157 prints 2:37
C     tcalc 2:37+54 prints 0:211=3:31
C     tcalc 2:37-1:54 prints 0:43=0:43
C     tcalc 1:54-12:37 prints -0:643=-10:43 or 13:17 yesterday
C
C     exit code meaning
C     --------- ------
C         0     all went well
C         1     no argument found
C         2     too many tokens
C         3     tokenization failed
C
C     variable  meaning
C     --------  -------
C     ARGS      concatenation of command-line arguments
C     BTD       routine converts an INTEGER*4 to numerals
C     DTB       routine converts numerals to an INTEGER*4
C     EXIT      system routine stops program with a return code
C     GETRGS    routine returns concatenation of run-line parameters
C     HRS       an hours value
C     I         index on tokens
C     INTINS    routine inserts an integer in a character string
C     K         index on the characters of a string
C     KR        index on the characters of RESULT
C     L         length of the parameter concatenation string
C     LENGTH    function returns index of last nonblank in a string
C     LH        length of an hours string
C     LR        length of RESULT
C     LTK       length of a token
C     MIN       a minutes value
C     MXLT      maximum allowed length of a token
C     MXNT      maximum allowed number of tokens
C     NOP       number of operators in concatenation
C     NTK       number of tokens in concatenation
C     OPS       operators in concatenation
C     PACKL     routine removes leading and embedded blanks
C     RC        return code; 0 => all went well
C     RESULT    output as a character string
C     SHIFTL    routine removes leading blanks from a string
C     SIGN      algebraic sign of net sum of minutes
C     SPACEL    routine removes leading and extra blanks
C     SUM       a net sum of minutes
C     TKNIZE    routine lists blank-separated tokens in a string
C     TKNS      the tokens found in ARGS
C     TMRW      " or hhhhhhhhhhh:mmmmmmmmmmm tomorrow"
C     YEST      " or hhhhhhhhhhh:mmmmmmmmmmm yesterday"
C
C     prepare to get the argument string concatenation
      CHARACTER*1 ARGS(256)
C
C     prepare to extract the operators and tokens
      PARAMETER(MXNT=10,MXLT=7)
      CHARACTER*1 OPS(MXNT),TKNS(MXLT,MXNT)
C
C     prepare to construct the output string
      CHARACTER*1 RESULT(80)/80*' '/,SIGN
      CHARACTER*1 YEST(37)/' ','o','r',' ','h','h','h','h','h','h',
     ;                     'h','h','h','h','h',':','m','m','m','m',
     ;                     'm','m','m','m','m','m','m',' ','y','e',
     ;                     's','t','e','r','d','a','y'/
      CHARACTER*1 TMRW(36)/' ','o','r',' ','h','h','h','h','h','h',
     ;                     'h','h','h','h','h',':','m','m','m','m',
     ;                     'm','m','m','m','m','m','m',' ','t','o',
     ;                     'm','o','r','r','o','w'/
      INTEGER*4 HRS,MIN,SUM,RC
C
C ------------------------------------------------------------------
C
C     get the expression to evaluate
      CALL GETRGS(ARGS,L)
      IF(L.EQ.0) CALL EXIT(1)
C
C     extract the operators
      IF(ARGS(1).EQ.'+') THEN
         OPS(1)='+'
         ARGS(1)=' '
         CALL SHIFTL(ARGS,L)
         L=L-1
      ELSEIF(ARGS(1).EQ.'-') THEN
         OPS(1)='-'
         ARGS(1)=' '
         CALL SHIFTL(ARGS,L)
         L=L-1
      ELSE
         OPS(1)='+'
      ENDIF
      NOP=1
      DO 1 K=1,L
           IF(ARGS(K).EQ.'+' .OR. ARGS(K).EQ.'-') THEN
              NOP=NOP+1
              IF(NOP.GT.MXNT) CALL EXIT(2)
              OPS(NOP)=ARGS(K)
              ARGS(K)=' '
           ENDIF
    1 CONTINUE
C
C     extract the tokens
      CALL TKNIZE(ARGS,L,MXLT,MXNT, TKNS,NTK,RC)
      IF(RC.NE.0) CALL EXIT(3)
      IF(NTK.GT.1) GO TO 2
C
C     there is one token; convert its format
      LTK=LENGTH(TKNS(1,1),MXLT)
      DO 3 K=1,LTK
           IF(TKNS(K,1).EQ.':') THEN
C             the token is a time in HH:MM format
              IF(K.EQ.1) THEN
C                the token looks like :MM
                 TKNS(1,1)=' '
                 HRS=0
                 GO TO 4
              ELSE
C                the token looks like H:MM or HH:MM
                 CALL DTB(TKNS(1,1),K-1, HRS,RC)
              ENDIF
    4         CALL DTB(TKNS(K+1,1),LTK-K, MIN,RC)
              MIN=MIN+60*HRS
              CALL BTD(MIN, RESULT,80,RC)
              IF(OPS(1).EQ.'-') RESULT(1)='-'
              RESULT(2)='0'
              RESULT(3)=':'
              CALL PACKL(RESULT,80)
              LR=LENGTH(RESULT,80)
              WRITE(6,901) (RESULT(KR),KR=1,LR)
  901         FORMAT(80A1)
              CALL EXIT(0)
           ENDIF
    3 CONTINUE
C     the token is a number of minutes
      CALL DTB(TKNS(1,1),LTK, MIN,RC)
      HRS=MIN/60
      MIN=MIN-60*HRS
      CALL BTD(HRS, RESULT,80,RC)
      IF(OPS(1).EQ.'-') RESULT(1)='-'
      CALL PACKL(RESULT,80)
      LH=LENGTH(RESULT,80)
      RESULT(LH+1)=':'
      CALL BTD(MIN, RESULT(LH+2),78-LH,RC)
      CALL PACKL(RESULT,80)
      LR=LENGTH(RESULT,80)
      WRITE(6,902) (RESULT(KR),KR=1,LR)
  902 FORMAT(11A1)
      CALL EXIT(0)
C
C     there are two or more tokens; compute their sum
    2 SUM=0
      DO 5 I=1,NTK
C          translate the hours:minutes token to minutes
           LTK=LENGTH(TKNS(1,I),MXLT)
           DO 6 K=1,LTK
                IF(TKNS(K,I).EQ.':') THEN
C                  the token is a time in HH:MM format
                   IF(K.EQ.1) THEN
C                     the token looks like :MM
                      TKNS(1,I)=' '
                      HRS=0
                      GO TO 7
                   ELSE
C                     the token looks like H:MM or HH:MM
                      CALL DTB(TKNS(1,I),K-1, HRS,RC)
                   ENDIF
    7              CALL DTB(TKNS(K+1,I),LTK-K, MIN,RC)
                   MIN=MIN+60*HRS
                   GO TO 8
                ENDIF
    6      CONTINUE
C
C          the token is already a number of minutes
           CALL DTB(TKNS(1,I),LTK, MIN,RC)
C
C          increment the total number of minutes
    8      IF(OPS(I).EQ.'+') SUM=SUM+MIN
           IF(OPS(I).EQ.'-') SUM=SUM-MIN
    5 CONTINUE
C
C     find the pieces needed to report result both ways
      IF(SUM.GE.0) THEN
         SIGN='+'
      ELSE
         SIGN='-'
         SUM=-SUM
      ENDIF
      HRS=SUM/60
      MIN=SUM-60*HRS
C
C     edit the output string
      CALL BTD(SUM, RESULT,80,RC)
      IF(SIGN .EQ. '-') RESULT(1)='-'
      RESULT(2)='0'
      RESULT(3)=':'
      CALL PACKL(RESULT,80)
      LR=LENGTH(RESULT,80)
      RESULT(LR+1)='='
      IF(SIGN .EQ. '-') RESULT(LR+2)='-'
      CALL BTD(HRS, RESULT(LR+3),78-LR,RC)
      CALL PACKL(RESULT,80)
      LR=LENGTH(RESULT,80)
      RESULT(LR+1)=':'
      CALL BTD(MIN, RESULT(LR+2),79-LR,RC)
      IF(RESULT(79).EQ.' ') RESULT(79)='0'
      CALL PACKL(RESULT,80)
      LR=LENGTH(RESULT,80)
C
C     handle results that fall in a different day
      IF(SIGN .EQ. '-') THEN
C        the result is negative; find the time yesterday
         SUM=1440-SUM
         HRS=SUM/60
         MIN=SUM-60*HRS
         DO 9 K=1,37
              RESULT(LR+K)=YEST(K)
    9    CONTINUE
         CALL INTINS(RESULT,80,HRS,LR+5,0, RESULT,LR)
         CALL INTINS(RESULT,80,MIN,LR-20,0, RESULT,LR)
         IF(RESULT(LR-11).EQ.' ') RESULT(LR-11)='0'
         CALL SPACEL(RESULT(LR-20),100-LR)
         CALL SPACEL(RESULT(LR-32),112-LR)
         LR=LENGTH(RESULT,80)
      ELSE
C        if the result is > 24:00 find the time tomorrow
         IF(SUM .GT. 1440) THEN
            SUM=SUM-1440
            HRS=SUM/60
            MIN=SUM-60*HRS
            DO 10 K=1,36
                  RESULT(LR+K)=TMRW(K)
   10       CONTINUE
            CALL INTINS(RESULT,80,HRS,LR+5,0, RESULT,LR)
            CALL INTINS(RESULT,80,MIN,LR-19,0, RESULT,LR)
            IF(RESULT(LR-10).EQ.' ') RESULT(LR-10)='0'
            CALL SPACEL(RESULT(LR-19),99-LR)
            CALL SPACEL(RESULT(LR-31),111-LR)
            LR=LENGTH(RESULT,80)
         ENDIF
      ENDIF
C
C     report the result
      WRITE(6,903) (RESULT(K),K=1,LR)
  903 FORMAT(80A1)
      CALL EXIT(0)
      END
